const Lang		= imports.lang
const Main		= imports.ui.main
const PopupMenu	= imports.ui.popupMenu
const St		= imports.gi.St
const UserMenu	= imports.ui.userMenu

const StatusButton = new Lang.Class({
	Name: 'StatusButton',
	Extends: UserMenu.UserMenuButton,
	
	_init: function() {
		this.parent()
		this.actor.label_actor.destroy()
		
		this._filter_IMMenu()
		this._filter_QBMenu()
		
		// Quit Button
		let userMenu = Main.panel._statusArea.userMenu
		userMenu._quitIcon = new St.Icon({'icon-name': 'system-shutdown', 'icon-size':16})
		userMenu.actor.get_children()[0].add(userMenu._quitIcon)
		
		userMenu._iconBox.hide()
		// compatibility with ‘Notifications Alert’ extension https://extensions.gnome.org/extension/258/notifications-alert-on-user-menu/
		this._originalIconBox = userMenu._iconBox
		userMenu._iconBox = this._iconBox
		if (this._originalIconBox.has_style_class_name('has-message-count-style')) this._iconBox.add_style_class_name('has-message-count-style')
	},
	
	_filter_IMMenu: function() {
		let items = this.menu._getMenuItems()
		
		this._updateHaveShutdown = function(){}			// for the ‘Suspend’/‘Shutdown’ entry not to generate Clutter-CRITICAL on mouseover
		let i
		for (i=2; i<items.length; i++) items[i].destroy()
		
		// Combobox
		items[0].removeActor(items[0]._children[0].actor)
		items[0]._children[0].actor.remove_actor(items[0]._children[0].actor.get_children()[0])		// not the same _children[0]
		items[0]._children[0].actor.get_children()[0].set_style("padding:0 2.5em;")
	},
	
	_filter_QBMenu: function() {
		let userMenu = Main.panel._statusArea.userMenu
		let items = userMenu.menu.box.get_children()
		this.hiddenItems = new Array()
		
		userMenu._statusChooser.actor.hide()
		userMenu._notificationsSwitch.actor.hide()
		
		let i
		for (i=2; i<items.length; i++) {
			let item = items[i]
			let label = item._delegate.label
			if (label != undefined && _(label.get_text()) == _("System Settings")) break
			
			item.reparent(this.menu.box)
			item._delegate.connect('activate', Lang.bind(this, function() { this.menu.close() }))
			
			this.hiddenItems.push([item,i])
		}
	},
	
	cleanAllYouCan: function() {
		let userMenu = Main.panel._statusArea.userMenu
		
		// unfilter QBMenu
		userMenu._statusChooser.actor.show()
		userMenu._notificationsSwitch.actor.show()
		
		this.hiddenItems.forEach(function(i) {
			Main.panel._statusArea.social.menu.box.remove_actor(i[0])
			userMenu.menu.addMenuItem(i[0]._delegate, i[1])
		})
		
		// restore IM Icon
		userMenu._quitIcon.destroy()
		if (this._iconBox.has_style_class_name('has-message-count-style')) this._originalIconBox.add_style_class_name('has-message-count-style')
		else this._originalIconBox.remove_style_class_name('has-message-count-style')
		userMenu._iconBox = this._originalIconBox
		
		let [presence, s, msg] = this._accountMgr.get_most_available_presence();		 // userMenu._iconBox is not initialised if
		userMenu._updatePresenceIcon(userMenu._accountMgr, presence, s, msg);			// the extension was enabled at Shell start
		
		userMenu._iconBox.show()
	}
})

function init() {}

let _IMButton
function enable() {
	_IMButton = new StatusButton
	
	let role = Main.panel._statusArea.userMenu.actor._rolePosition
	Main.panel._statusArea.userMenu.actor._rolePosition = role + 1
	
	Main.panel.addToStatusArea('social', _IMButton, role)
}
function disable() {
	_IMButton.cleanAllYouCan()
	_IMButton.destroy()
	
	// Not re_rolePositionning userMenu, the order may have changed
}
